@@ -1,16 +1,17 @@
 # -*- coding: utf-8 -*-
 # Part of the scorify package
-# Copyright 2014 Board of Regents of the University of Wisconsin System
+# Copyright (c) 2020 Board of Regents of the University of Wisconsin System
+from __future__ import absolute_import
 
-import mappings
-import aggregators
+from scorify import mappings
+from scorify import aggregators
 
 """
 Directives define operations we'll be performing on our input data files.
 A scoresheet defines a set of directives; directive will be run on the input.
 
 Examples include:
-layout: Defines the header row and any rows we'll want to skip
+layout: Defines the header row and any rows we'll want to skip or keep
 
 exclude: Defines any rows we'll want to exclude from processing
 
@@ -44,7 +45,7 @@ class Layout(object):
 
     def __init__(self, info):
         info_lower = info.strip().lower()
-        if info_lower not in set(['header', 'data', 'skip']):
+        if info_lower not in set(['header', 'data', 'skip', 'keep']):
             raise DirectiveError(
                 "Didn't understand layout {0!r}".format(info))
 
@@ -52,6 +53,37 @@ class Layout(object):
         super(Layout, self).__init__()
 
 
+class Rename(object):
+    """
+    Renames a column. Renamed columns must be referred to by the new name.
+    Does not allow giving a 0-length name. Empty names or names will be
+    rejected, too. Sorry if your column names are blank.
+
+    All logic for this is in the datafile, since that's the only place where
+    we know column names.
+    """
+    def __init__(self, original_name, new_name):
+        self.original_name = str(original_name)
+        self.new_name = str(new_name)
+        if len(self.original_name) == 0:
+            raise DirectiveError("Can't rename a blank column.")
+        if len(self.new_name) == 0:
+            raise DirectiveError("Can't give a column a blank name.")
+        if self.original_name == self.new_name:
+            raise DirectiveError(
+                "Can't rename a column to its original name.")
+
+    def conflicts_with(self, other):
+        return (
+            self.original_name == other.original_name or
+            self.original_name == other.new_name or
+            self.new_name == other.new_name or
+            self.new_name == other.original_name)
+
+    def __str__(self):
+        return "rename %s %s" % (self.original_name, self.new_name)
+
+
 class Exclude(object):
     """
     Skips a line in the data based on a column being equal to some value. So:
@@ -97,19 +129,23 @@ class Score(object):
     to specify what measure the column is used in and what transform should
     be applied to it. Examples:
 
-    score PARTICIPANT
-    score DATA_COL_1 MEASURE_1 normal
-    score DATA_COL_2 MEASURE_1 reverse
-
+        score PARTICIPANT
+        score DATA_COL_1 MEASURE_1 normal
+        score DATA_COL_2 MEASURE_1 reverse
 
     If the measure name is blank, the column doesn't get added to any measure.
     If the transform is blank, the data is copied without any changes.
+
+    The fifth column is an optional name, for example
+
+        score DATA_COLUMN MEASURE_1 reverse AwesomeReversed
     """
 
-    def __init__(self, column, measure_name, transform=None):
+    def __init__(self, column, measure_name, transform=None, output_name=None):
         self.column = column
         self.measure_name = measure_name
         self.transform = transform
+        self.output_name = output_name
         super(Score, self).__init__()
 
 
@@ -128,8 +164,9 @@ class Measure(object):
             fname, fx, to_use = aggregators.parse_expr(aggregation_expr)
             self.agg_fx = fx
             self.to_use = to_use
+
         except aggregators.AggregatorError as exc:
-            raise DirectiveError(exc.message)
+            raise DirectiveError(str(exc))
 
         super(Measure, self).__init__()
 
