@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 # Part of the scorify package
-# Copyright 2014 Board of Regents of the University of Wisconsin System
+# Copyright (c) 2020 Board of Regents of the University of Wisconsin System
 
-from __future__ import division
+from __future__ import division, absolute_import
 import re
 
 """
@@ -33,6 +33,8 @@ class Mapping(object):
             return LinearMapping.from_string(fx_string)
         if fx_string.find("discrete_map(") == 0:
             return DiscreteMapping.from_string(fx_string)
+        if fx_string.find("passthrough_map(") == 0:
+            return PassthroughMapping.from_string(fx_string)
 
 
 class Identity(Mapping):
@@ -110,18 +112,16 @@ class DiscreteMapping(Mapping):
         self.map_dict = map_dict
 
     def transform(self, value):
-        if value in self.map_dict:
-            return self.map_dict[value]
         # In the case where there's no match, it's probably best to return an
         # empty string. They'll always see the original value in the output
         # anyhow.
-        return ''
+        return self.map_dict.get(value, '')
 
     @classmethod
     def from_string(kls, fx_string):
         def ues(s):
             # Unescapes strings -- will turn '\"' into '"'
-            return bytes(s).decode('unicode-escape')
+            return bytes(s, 'utf-8').decode('unicode_escape')
         # Python 2.7 has a sweet dict comprehension, but I'll keep 2.5 compat
         result = dict((
             (ues(match.group(1)), ues(match.group(2)))
@@ -129,5 +129,13 @@ class DiscreteMapping(Mapping):
         return kls(result)
 
 
+class PassthroughMapping(DiscreteMapping):
+    def __init__(self, map_dict):
+        super(PassthroughMapping, self).__init__(map_dict)
+
+    def transform(self, value):
+        return self.map_dict.get(value, value)
+
+
 class MappingError(ValueError):
     pass
