@@ -1,25 +1,30 @@
 # -*- coding: utf-8 -*-
 # Part of the scorify package
-# Copyright 2014 Board of Regents of the University of Wisconsin System
+# Copyright (c) 2020 Board of Regents of the University of Wisconsin System
+from __future__ import absolute_import
 
 from collections import defaultdict
 from scorify.errors import HaystackError
 
-NAN = float('nan')
+NaN = float('nan')
 
 
 class ScoredData(object):
-    def __init__(self, header=None, data=None, measure_columns=None):
+    def __init__(self, header=None, keep=None, data=None, measure_columns=None):
         self.header = header or []
         self.data = data or []
+        self.keep = keep or []
         self.measure_columns = measure_columns or defaultdict(list)
 
     def columns_for(self, measure_list):
         out = []
         for name in measure_list:
-            if name not in self.measure_columns:
+            if name in self.measure_columns:
+                out.extend(self.measure_columns[name])
+            elif name in self.header:
+                out.append(name)
+            else:
                 raise KeyError(name)
-            out.extend(self.measure_columns[name])
         return out
 
     def known_measures(self):
@@ -35,9 +40,13 @@ class ScoredData(object):
 class Scorer(object):
     @classmethod
     def score_name(kls, directive):
+        if directive.output_name:
+            return directive.output_name
         name = directive.column
-        if len(directive.measure_name) > 0:
+        if directive.measure_name != "None" and len(directive.measure_name) > 0:
             name += ': ' + directive.measure_name
+        if len(directive.transform) > 0:
+            name += ': ' + directive.transform
         return name
 
     @classmethod
@@ -53,6 +62,18 @@ class Scorer(object):
         out.header = [kls.score_name(d) for d in score_section.directives]
         out.measure_columns = kls.make_measure_columns(score_section)
 
+        for k in datafile.keep:
+            kept = {}
+            for d in score_section.directives:
+                name = kls.score_name(d)
+                try:
+                    kept[name] = k[d.column]
+                except KeyError as err:
+                    # No kept data for this column? No worries, just blank is fine
+                    kept[name] = ''
+
+            out.keep.append(kept)
+
         for r in datafile.data:
             scored = {}
             for s in score_section.directives:
@@ -60,7 +81,7 @@ class Scorer(object):
                     tx = transform_section[s.transform]
                 except KeyError as exc:
                     raise TransformError(
-                        "transforms", exc.message,
+                        "transforms", "Key not found: " + s.transform,
                         transform_section.known_transforms())
 
                 name = kls.score_name(s)
@@ -68,10 +89,10 @@ class Scorer(object):
                     sval = tx.transform(r[s.column])
                 except KeyError as err:
                     raise ScoringError(
-                        "data columns", err.message,
+                        "data columns", str(err),
                         datafile.header)
                 except ValueError:
-                    sval = NAN
+                    sval = NaN
                 scored[name] = sval
             out.data.append(scored)
 
@@ -87,12 +108,12 @@ class Scorer(object):
                     cols = scored_data.columns_for(m.to_use)
                 except KeyError as exc:
                     raise AggregationError(
-                        "measures", exc.message, scored_data.known_measures())
+                        "measures", str(exc), scored_data.known_measures())
                 vals = [row[col] for col in cols]
                 try:
                     row[m.name] = m.agg_fx(vals)
                 except ValueError:
-                    row[m.name] = NAN
+                    row[m.name] = NaN
 
 
 class TransformError(HaystackError):
